<?php
class sv43v_application_form_fields extends wv43v_action {
	private $_form = null;
	public function __construct($application, $form) {
		parent::__construct ( $application );
		$this->_form = $form;
	}
	public function &field($field) {
		return $this->cache ( 'sv43v_application_form_fields_field', array ($this->_form, $field ), true );
	}
	public function &data() {
		return parent::data ( $this->_form );
	}
	public function front_html() {
		$html = "";
		foreach ( $this->data ()->field_definitions as $field => $definition ) {
			$html .= $this->field ( $field )->front_html ();
		}
		return $html;
	}
	public function table_data() {
		$data = array ();
		$questions = null;
		$correct = 0;
		foreach ( $this->data ()->field_definitions as $field => $definition ) {
			if ($definition ['table'] ['include'] == 1) {
				$name = $definition ['table'] ['name'];
				if ($name == "") {
					$name = $definition ["question"];
					$this->table ()->field_name ( $name );
				}
				switch ($definition ['type']) {
					case 'checkbox' :
						if (is_array ( $definition ['options'] )) {
							foreach ( $definition ['options'] as $opt ) {
								$val = 0;
								if (in_array ( $opt, ( array ) $definition ['value'] )) {
									$val = 1;
								}
								$data [$name . '[' . $opt . ']'] = $val;
							}
						} else {
							$data [$name] = $definition ['value'];
						}
						break;
					case 'quiz_question' :
						$data [$name] = $definition ['value'];
						if ($definition ['value'] == $definition ['answer']) {
							$correct ++;
						}
						$questions ++;
						break;
					case 'link' :
						$data [$name . "[url]"] = $definition ['value'] ['url'];
						$data [$name . "[desc]"] = $definition ['value'] ['desc'];
						break;
					case 'formsent' :
						$data [$name] = $definition ['value'];
						$data ['info[form_received]'] = $this->table ()->date ();
						break;
					default :
						$data [$name] = $definition ['value'];
						break;
				}
			}
		}
		if (null !== $questions) {
			$data ['info[quiz_questions]'] = $questions;
			$data ['info[quiz_score]'] = $correct;
		}
		$data ['info[submission]'] = 0;
		$this->table ()->addslashes ( $data );
		return $data;
	}
	private function is_email($email) {
		$email = trim ( $email );
		$return = false;
		if (! empty ( $email )) {
			$return = true;
		}
		return $return;
	}
	public function error() {
		$return = 0;
		foreach ( $this->data ()->field_definitions as $field ) {
			if ($field ['error'] != 0 ) {
				$return = $field ['error'];
				break;
			}
		}
		return $return;
	}
	public function first_field($field, $default = null) {
		$return = $default;
		$subjects = $this->get_questions ( array ($field ) );
		foreach ( $subjects as $subject ) {
			if (trim ( $subject->$field ) != '') {
				$return = trim ( $subject->$field );
				break;
			}
		}
		return $return;
	}
	public function email_details() {
		$return = $this->get_types ( array ('email', 'name', 'cc' ) );
		foreach ( $return as $key => &$value ) {
			if (! $this->is_email ( $value->email ['value'] )) {
				unset ( $return [$key] );
			} else {
				$value->email ['value'] = trim ( $value->email ['value'] );
				//$value->email_question = $value->question;
				if (trim ( $value->name ['value'] ) == '') {
					$value->name ['value'] = $value->email ['value'];
				}
				$value->name ['value'] = trim ( $value->name ['value'] );
				$value->cc ['value'] = (! empty ( $value->cc ['value'] ));
			}
		}
		unset ( $value );
		return $return;
	}
	public function quiz_details() {
		$questions = $this->get_types ( array ('quiz_question' ) );
		$results = new stdClass ();
		$results->questions = count ( $questions );
		$results->correct = 0;
		foreach ( $questions as $key => $value ) {
			if ($value->quiz_question ['details'] ['value'] == $value->quiz_question ['details'] ['answer']) {
				$results->correct ++;
			}
		}
		return $results;
	
	}
	public function poll_details() {
		$return = $this->get_types ( array ('poll_question' ) );
		return $return;
	
	}
	public function email_txt() {
		$html = array ();
		foreach ( $this->email_details () as $sender ) {
			$html [] = "Submitted by {$sender->name['value']} <{$sender->email['value']}>";
			break;
		}
		foreach ( array_keys ( $this->data ()->field_definitions ) as $field ) {
			$returned = $this->field ( $field )->email_txt ();
			if (! empty ( $returned )) {
				$html [] = $returned;
			}
		}
		$html [] = "--\nThis mail is sent via " . get_the_title () . " on " . get_bloginfo ( 'name' ) . " " . get_bloginfo ( 'url' );
		$html = implode ( "\n", $html );
		return $html;
	}
	public function post_html() {
		$html = array ();
		foreach ( array_keys ( $this->data ()->field_definitions ) as $field ) {
			$returned = $this->field ( $field )->post_html ();
			if (! empty ( $returned )) {
				$html [] = $returned;
			}
		}
		$html = implode ( "<br/>", $html );
		return $html;
	}
	public function results_html() {
		$html = array ();
		foreach ( array_keys ( $this->data ()->field_definitions ) as $field ) {
			$returned = $this->field ( $field )->results_html ();
			if (! empty ( $returned )) {
				$html [] = $returned;
			}
		}
		$quiz_detials = $this->quiz_details ();
		if ($quiz_detials->questions > 0) {
			$score = "You scored %d out of a possible %d <hr/>";
			$score = sprintf ( $score, $quiz_detials->correct, $quiz_detials->questions );
			array_unshift ( $html, $score );
		}
		$html = implode ( "<br/>", $html );
		return $html;
	}
	public function get_types($types = array()) {
		$return = array ();
		$max = 0;
		foreach ( $types as $type ) {
			$return [$type] = array ();
			foreach ( $this->data ()->field_definitions as $key => $value ) {
				if ($type == $value ['type']) {
					$return [$type] [] = array ('value' => $value ['value'], 'question' => $value ['question'], 'details' => $value );
				}
			}
			$max = max ( $max, count ( $return [$type] ) );
		}
		foreach ( $types as $type ) {
			$return [$type] = array_pad ( $return [$type], $max, '' );
		}
		$data = array ();
		for($cnt = 0; $cnt < $max; $cnt ++) {
			$data [$cnt] = new stdClass ();
			foreach ( $types as $type ) {
				$data [$cnt]->$type = $return [$type] [$cnt];
			}
		}
		return $data;
	}
	public function get_questions($questions = array()) {
		$return = array ();
		$max = 0;
		foreach ( $questions as $question ) {
			$return [$question] = array ();
			foreach ( $this->data ()->field_definitions as $key => $value ) {
				if (stripos ( $value ['question'], $question ) === 0) {
					$return [$question] [] = $value ['value'];
				}
			}
			$max = max ( $max, count ( $return [$question] ) );
		}
		foreach ( $questions as $question ) {
			$return [$question] = array_pad ( $return [$question], $max, '' );
		}
		$data = array ();
		for($cnt = 0; $cnt < $max; $cnt ++) {
			$data [$cnt] = new stdClass ();
			foreach ( $questions as $question ) {
				$data [$cnt]->$question = $return [$question] [$cnt];
			}
		}
		return $data;
	}
	public function admin_html() {
		$html = "";
		foreach ( array_keys ( $this->data ()->field_definitions ) as $field ) {
			$html .= $this->field ( $field )->admin_html ();
		}
		return $html;
	}
}